/**
 * Dijjer
 * ======
 *
 * Version: 0.1.0
 *
 * Author: Christopher Ottley
 *
 * Description: The Dijjer extension for Firefox and Mozilla integrates Dijjer web cache functionality in the browser
 *
 *  Copyright (c) 2004 Christopher Ottley.
 *
 *  This file is part of Dijjer.
 *
 *  Dijjer is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dijjer is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with MAF; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// Provides a dijjer: protocol handler

const dijjerProtocolScheme = "dijjer";
const dijjerProtocolName = "Dijjer Access Protocol";
const dijjerProtocolContractID = "@mozilla.org/network/protocol;1?name=" + dijjerProtocolScheme;
const dijjerProtocolCID = Components.ID("fbe03a9e-47fc-40fd-b840-e41eff9fd8f3");

function DijjerProtocol() { }

DijjerProtocol.prototype = {
  QueryInterface: function(iid)   {
    if (!iid.equals(Components.interfaces.nsIProtocolHandler) &&
        !iid.equals(Components.interfaces.nsISupports)) {
      throw Components.results.NS_ERROR_NO_INTERFACE;
    }
    return this;
  },

  scheme: dijjerProtocolScheme,
  defaultPort: -1,
  protocolFlags: Components.interfaces.nsIProtocolHandler.URI_NORELATIVE |
                 Components.interfaces.nsIProtocolHandler.URI_NOAUTH,

  allowPort: function(port, scheme) {
    return false;
  },

  newURI: function(spec, charset, baseURI) {
    var uri = Components.classes["@mozilla.org/network/simple-uri;1"]
                .createInstance(Components.interfaces.nsIURI);
    uri.spec = spec;
    return uri;
  },

  newChannel: function(aURI) {
    var ios = Components.classes["@mozilla.org/network/io-service;1"]
                 .getService(Components.interfaces.nsIIOService);

    var strURI = aURI.spec;

    if (strURI.toLowerCase().startsWith("dijjer:")) {
      requestURI = strURI.substring(7, strURI.length);
      return ios.newChannel("http://127.0.0.1:9115/" + requestURI, null, null);
    } else {
      // Er, not a dijjer protocol. Why are we being invoked?
      return ios.newChannel("about:blank", null, null);
    }
  }

}

function dijjerdebug(text) {
  Components.classes["@mozilla.org/consoleservice;1"]
    .getService(Components.interfaces.nsIConsoleService)
    .logStringMessage(text);
};

String.prototype.startsWith = function(needle) {
  return (this.substring(0, needle.length) == needle);
};

var DijjerProtocolFactory = new Object();

DijjerProtocolFactory.createInstance = function (outer, iid) {
  if (outer != null) {
    throw Components.results.NS_ERROR_NO_AGGREGATION;
  }

  if (!iid.equals(Components.interfaces.nsIProtocolHandler) &&
      !iid.equals(Components.interfaces.nsISupports)) {
    throw Components.results.NS_ERROR_NO_INTERFACE;
  }

  return new DijjerProtocol();
}


/**
 * XPCOM component registration
 */
var DijjerProtocolModule = new Object();

DijjerProtocolModule.registerSelf = function (compMgr, fileSpec, location, type) {
  compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
  compMgr.registerFactoryLocation(dijjerProtocolCID,
                                  dijjerProtocolName,
                                  dijjerProtocolContractID,
                                  fileSpec,
                                  location,
                                  type);
}

DijjerProtocolModule.getClassObject = function(compMgr, cid, iid) {
  if (!cid.equals(dijjerProtocolCID)) {
    throw Components.results.NS_ERROR_NO_INTERFACE;
  }

  if (!iid.equals(Components.interfaces.nsIFactory)) {
    throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
  }

  return DijjerProtocolFactory;
}

DijjerProtocolModule.canUnload = function (compMgr) {
  return true;
}

function NSGetModule(compMgr, fileSpec) {
  return DijjerProtocolModule;
}
