/**
 * Dijjer
 * ======
 *
 * Version: 0.2.0
 *
 * Author: Christopher Ottley
 *
 * Description: The Dijjer extension for Firefox and Mozilla integrates Dijjer web cache functionality in the browser
 *
 *  Copyright (c) 2005 Christopher Ottley.
 *
 *  This file is part of Dijjer.
 *
 *  Dijjer is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dijjer is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with MAF; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 *
 * Release 0.2.0
 *
 * Updated code from Dijjer CVS repository.
 * Added TextArea display for Applet.
 * Added Applet Logger that sends output to TextArea.
 * Applet is now signed using a cert from certum.pl.
 *
 *
 * Initial Release 0.1.0
 *
 * dijjer: protocol redirects users to http://127.0.0.1:9115/
 * URLs on web pages with http://127.0.0.1:9115/ or http://dij.dijjer.org:9115/ are replaced with with dijjer:
 *   when the pages are loaded
 * Modified function so that URLs starting with http://dijjer.org/get/ are replaced with dijjer:
 * Added ability to show java console from Dijjer menu.
 *
 */

var browserWindow = window;

function dijjer() {

};

dijjer.prototype = {


  onWindowLoad: function(event) {
    if (event.originalTarget == "[object XULDocument]") {
      // New window

      // Get hidden window
      var appShell = Components.classes["@mozilla.org/appshell/appShellService;1"]
                        .getService(Components.interfaces.nsIAppShellService);
      var hiddenWnd = appShell.hiddenDOMWindow;

      if (typeof(hiddenWnd.dijjerloaded) == "undefined") {
        hiddenWnd.dijjerloaded = true;

        var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                        .getService(Components.interfaces.nsIPrefService).getBranch("dijjer.");

        DijjerPostSetup.complete();
      }
    } else {
      if (event.originalTarget == "[object HTMLDocument]") {
        // New tab

        // We have some work to do
        var links = event.originalTarget.links;

        for (var j=0; j < links.length; j++) {
          if (links[j].href.toLowerCase().startsWith("http://127.0.0.1:9115/") ) {
            links[j].href="dijjer:" + links[j].href.substring(22, links[j].href.length);
          } else if (links[j].href.toLowerCase().startsWith("http://dij.dijjer.org:9115/") ) {
            links[j].href="dijjer:" + links[j].href.substring(27, links[j].href.length);
          } else if (links[j].href.toLowerCase().startsWith("http://dijjer.org/get/") ) {
            links[j].href="dijjer:" + links[j].href.substring(22, links[j].href.length);
          }
        }
      }
    }
  },

  onWindowClose: function(event) {

  },

  onWindowFocus: function(event) {

  },

  QueryInterface: function(iid) {

    if (!iid.equals(Components.interfaces.nsISupports)) {
      throw Components.results.NS_ERROR_NO_INTERFACE;
    }

    return this;
  }

};


var DijjerAppletProxy = {
  appletchromeurl: "chrome://dijjer/content/dijjerAppletDLG.xul",

  isDijjerRunning: function() {
    return isWindowOpen(this.appletchromeurl);
  },

  /**
   * Returns true if a window with that id is open
   */
  isWindowOpen: function(needleLocation) {
    var result = false;

    try {
      var wmI = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                   .getService(Components.interfaces.nsIWindowMediator);
      var entries = wmI.getEnumerator(null);

      while (entries.hasMoreElements()) {
        currWindow = entries.getNext();
        if (currWindow.location == needleLocation) {
          result = true;
          break;
        }
      }
    } catch (e) {
      dijjerdebug(e);
    }

    return result;
  },

  startDijjer: function() {
    var url = this.appletchromeurl;

    var w = 370;
    var h = 175;

    var win_prefs = "chrome,dependent=no,resizable=yes,width="+ w +",height=" + h;
    window.openDialog(url, "_blank", win_prefs);
  },

  stopDijjer: function() {
    var needleLocation = this.appletchromeurl;

    window.getBrowser().loadURI("http://127.0.0.1:9115/shutdown?confirm=yes", null, null);

    // Close window by id? Hmmm...
    try {
      var wmI = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                   .getService(Components.interfaces.nsIWindowMediator);
      var entries = wmI.getEnumerator(null);

      while (entries.hasMoreElements()) {
        currWindow = entries.getNext();
        if (currWindow.location == needleLocation) {
          currWindow.close();
          break;
        }
      }
    } catch (e) {
      dijjerdebug(e);
    }
  },

  statusPage: function() {
    window.getBrowser().loadURI("http://127.0.0.1:9115/", null, null);
  },

  showJavaConsole: function() {
     Components.classes["@mozilla.org/oji/jvm-mgr;1"]
        .getService(Components.interfaces.nsIJVMManager).showJavaConsole();
  }
}

var DijjerPostSetup = {

  progid: "{1c7225fe-f321-4641-8ab6-e6db6031ec24}",

  postsetupversion: "0.2.0", // 0.2.0 Is the signed applet release

  complete: function() {

    // Get preference maf.postsetup.complete
    var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                   .getService(Components.interfaces.nsIPrefService).getBranch("dijjer.");

    try {
      var setupComplete = prefs.getBoolPref("postsetup." + this.postsetupversion + ".complete");
    } catch(e) { setupComplete = false; }

    if (!setupComplete) {
      // Make the dijjer directory
      this.makeDijjerDirectory();

      // Copy files
      this.copyFiles();

      // Set preference maf.postsetup.complete
      prefs.setBoolPref("postsetup." + this.postsetupversion + ".complete", true);
    }
  },

  /**
   * @returns A string representing the location of the user's profile directory
   */
  _getProfileDir: function() {
    try {
      var result = Components.classes["@mozilla.org/file/directory_service;1"]
                      .getService(Components.interfaces.nsIProperties)
                      .get("ProfD", Components.interfaces.nsIFile).path;
    } catch (e) {
      result = "";
    }
    return result;
  },

  /**
   * @returns an array of files in a directory
   */
  _getFilesList: function(sourcepath) {
    var result = new Array();

    var oDir = Components.classes["@mozilla.org/file/local;1"]
                  .createInstance(Components.interfaces.nsILocalFile);
    oDir.initWithPath(sourcepath);

    if (oDir.exists() && oDir.isDirectory()) {
      var entries = oDir.directoryEntries;

      while (entries.hasMoreElements()) {
        var currFile = entries.getNext();
        currFile.QueryInterface(Components.interfaces.nsILocalFile);

        result[result.length] = currFile.leafName;
      }
    }
    return result;
  },

  /**
   * Copy an individual file
   */
  _copyFile: function(source, dest) {
    try {
      // Make sure source exists and dest doesn't
      if (this.checkFileExists(source) && !this.checkFileExists(dest)) {
        this.copyBinaryFile(source, dest);
      }
    } catch(e) { dijjerdebug(e); }
  },

  /**
   * Returns true if the file in the path exists
   */
  checkFileExists: function(filePathToCheck) {
    var oFile = Components.classes["@mozilla.org/file/local;1"]
                  .createInstance(Components.interfaces.nsILocalFile);
    oFile.initWithPath(filePathToCheck);
    return oFile.exists();
  },

  /**
   * Read the contents of a file as bytes
   */
  readBinaryFile: function(str_Filename) {
    try {
      var obj_File = Components.classes["@mozilla.org/file/local;1"]
                        .createInstance(Components.interfaces.nsILocalFile);
      obj_File.initWithPath(str_Filename);

      var obj_InputStream = Components.classes["@mozilla.org/network/file-input-stream;1"]
                               .createInstance(Components.interfaces.nsIFileInputStream);
      obj_InputStream.init(obj_File, 0x01, 0444, null);

      var obj_BinaryIO = Components.classes["@mozilla.org/binaryinputstream;1"]
                            .createInstance(Components.interfaces.nsIBinaryInputStream);

      obj_BinaryIO.setInputStream(obj_InputStream);
    } catch (e) {
      dijjerdebug(e);
    }

    try {
      var str = obj_BinaryIO.readByteArray(obj_File.fileSize);

    } catch (e) {
      dijjerdebug(e);
    }
    obj_BinaryIO.close();
    obj_InputStream.close();

    return str;
  },


  /**
   * Create binary file
   */
  copyBinaryFile: function(source, fileToCreate) {
    try {
      var oFile = Components.classes["@mozilla.org/file/local;1"]
                     .createInstance(Components.interfaces.nsILocalFile);
      oFile.initWithPath(fileToCreate);
      if (!oFile.exists()) {
        oFile.create(0x00, 0644);
      }
    } catch (e) {
      dijjerdebug(e);
    }

    try {
      var oTransport = Components.classes["@mozilla.org/network/file-output-stream;1"]
                          .createInstance(Components.interfaces.nsIFileOutputStream);
      oTransport.init( oFile, 0x04 | 0x08 | 0x10, 064, 0 );

      var obj_BinaryIO = Components.classes["@mozilla.org/binaryoutputstream;1"]
                           .createInstance(Components.interfaces.nsIBinaryOutputStream);
      obj_BinaryIO.setOutputStream(oTransport);

      var contents = this.readBinaryFile(source);

      obj_BinaryIO.writeByteArray(contents, contents.length);
      oTransport.close();
    } catch (e) {
      dijjerdebug(e);
    }
  },

  /**
   * Cross platform append
   */
  appendToDir: function(initialDirectory, subDirectory) {
    var result = initialDirectory;
    try {
      var dir = Components.classes["@mozilla.org/file/local;1"]
                   .createInstance(Components.interfaces.nsILocalFile);
      dir.initWithPath(initialDirectory);
      dir.append(subDirectory);
      result = dir.path;
    } catch(e) {

    }
    return result;
  },

  /**
   * Create directory
   */
  createDir: function(dirToCreate) {
    var dir = null;
    try {
      dir = Components.classes["@mozilla.org/file/local;1"]
              .createInstance(Components.interfaces.nsILocalFile);
      dir.initWithPath(dirToCreate);

      // Make the directory!!!
      if (!dir.exists()) {
        dir.create(0x01, 0777);
      }
    } catch (e) {

    }
    return dir;
  },


  /**
   * Create the Dijjer directory structure.
   */
  makeDijjerDirectory: function() {
    try {
      var profileDir = this._getProfileDir();
      this.createDir(this.appendToDir(profileDir, "dijjer"));
    } catch(e) { dijjerdebug(e); }
  },


  /**
   * Copy a set of files from the extensions, lib directory
   * Only copy if FF 0.9 or higher
   */
  copyFiles: function() {
    try {

      var profileDir = this._getProfileDir();
      var sourceDir = profileDir;

      // If firefox 0.9 or higher
      var isFF09OrHigher = false;
      if ((navigator.vendor != null) && (navigator.vendorSub != null)) {
        if (navigator.vendor.toLowerCase() == "firefox") {
          isFF09OrHigher = (parseFloat(navigator.vendorSub.substring(navigator.vendorSub.indexOf(".") + 1,
                          navigator.vendorSub.length)) + (parseFloat(navigator.vendorSub.substring(0,
                          navigator.vendorSub.indexOf("."))) * 100) >= 9);
        }
      }

      if (isFF09OrHigher) {
        sourceDir = this.appendToDir(sourceDir, "extensions");
        sourceDir = this.appendToDir(sourceDir, this.progid);
        sourceDir = this.appendToDir(sourceDir, "lib");

        var destDir = this.appendToDir(profileDir, "dijjer");

        var filesList = this._getFilesList(sourceDir);
        for (var i=0; i<filesList.length; i++) {
          this._copyFile(this.appendToDir(sourceDir, filesList[i]), this.appendToDir(destDir, filesList[i]));
        }
      }

    } catch (e) {
      dijjerdebug(e);
    }

  }

}


try {

  /**
  * Main object
  */
  var Dijjer = new dijjer();

  browserWindow.addEventListener("close", Dijjer.onWindowClose, true);
  browserWindow.addEventListener("load", Dijjer.onWindowLoad, true);
  browserWindow.addEventListener("focus", Dijjer.onWindowFocus, true);

} catch(e) {
  dijjerdebug(e);
}

function dijjerdebug(text) {
  Components.classes["@mozilla.org/consoleservice;1"]
    .getService(Components.interfaces.nsIConsoleService)
    .logStringMessage(text);
};

String.prototype.trim = function() {
  // skip leading and trailing whitespace
  // and return everything in between
  var x=this;
  x=x.replace(/^\s*(.*)/, "$1");
  x=x.replace(/(.*?)\s*$/, "$1");
  return x;
};

String.prototype.replaceAll = function(needle, newneedle) {
  var x=this;
  x=x.split(needle).join(newneedle);
  return x;
};

String.prototype.startsWith = function(needle) {
  return (this.substring(0, needle.length) == needle);
};

// ====================
// Dijjeridoo code
// ====================

// prepends http://127.0.0.1:9115/ to the current URL
// based on Coralize 0.3 by Patrick Reynolds (http://www.cs.duke.edu/~reynolds/)

function Dijjeridoo(newtab) {
  var cm = gContextMenu;
  var url;
  if (cm.onLink) {
    url = cm.linkURL();
  }
  else {
    url = window._content.location;
  }

  var pre = "http://127.0.0.1:9115/";

  //alert(url);
  var myRe = new RegExp("^http://127\.0\.0\.1:9115/http://");
  var reResults = myRe.exec(url);
  if (reResults != null) {
    alert("\""+url+"\" is already Dijjeridooed.");
    return;
  }

  myRe = new RegExp("^([^:]+:)?/{2,}([^:/]+)(:\[0-9]+)?/(.*)");
  reResults = myRe.exec(url);
  //alert(reResults);
  if (reResults == null) {
    alert("\""+url+"\" cannot be Dijjeridooed.");
    return;
  }
  if (reResults[1] != "http:") {
    alert("Only HTTP urls can be Dijjeridooed.");
    return;
  }

  var newLoc;
  if (reResults[3] != null) {
    var port = new String(reResults[3]).substr(1);
    newLoc = pre+"http://"+reResults[2]+"."+port+"/"+reResults[4];
  }
  else {
  newLoc = pre+"http://"+reResults[2]+"/"+reResults[4];
  }

  if (newtab)
    gBrowser.addTab(newLoc);
  else
    window._content.location = newLoc;
}

window.addEventListener("load",corInit,false);

// hides the item when approproiate (use same logic as for Back, Stop etc.)
// script written by Stephen Clavering, used in his goHome extension, borrowed via BugMeNot

function corInit() {
  document.getElementById("contentAreaContextMenu").addEventListener("popupshowing",corHide,false);
}

function corHide() {
  var cm = gContextMenu;
  document.getElementById("dijjeridooThis").hidden = ( cm.isTextSelected || cm.onLink || cm.onImage || cm.onTextInput );
  document.getElementById("dijjeridooLink").hidden = ( cm.isTextSelected || cm.onTextInput || cm.onImage || !cm.onLink );
  document.getElementById("dijjeridooLinkTab").hidden = ( cm.isTextSelected || cm.onTextInput || cm.onImage || !cm.onLink );
}
